#include "StepperMotor.h"

extern uint8_t gMotor;
extern void sendMotor(void);

static const uint16_t msToStart = 50;
static const uint16_t msToHang = 50;
static const uint16_t stepTimeMS = 12;
static const uint8_t slowSpeed = static_cast<uint8_t>( stepTimeMS );
static const uint16_t homeStepTimeMS = 12;
static const uint8_t highSpeed = static_cast<uint8_t>( homeStepTimeMS );

StepperMotor::StepperMotor(const uint8_t enable, const uint8_t a, const uint8_t b, 
	const uint8_t led, const uint8_t opto) :
timeNextStateChange(0),
	bitEnable(enable),
	bitA(a),
	bitB(b),
	bitLEDPwr(led),
	pinOpto(opto),
	phase(0),
    desiredPosition(DoNothingDesiredPosition),
	count(0),
	position(0),
	homeCounter(0),
	bHomeHigh(false),
	state(E_IDLE)
{
	pinMode(pinOpto,INPUT);
}

void StepperMotor::goToIdle(void)
{
	bitSet(gMotor,bitEnable); // disable the motor
	//    bitClear(gMotor,bitEnable); // enable the motor!!
	bitClear(gMotor,bitA);
	bitClear(gMotor,bitB);
	bitClear(gMotor,bitLEDPwr);  // turn off LED power
	sendMotor();
	state = E_IDLE;
	digitalWrite(pinOpto,LOW);  // reduce open drain voltage 
}

void StepperMotor::setMotorBitsFromPhase(void)
{
	// do the quadrature mapping.
	uint8_t a = phase;
	if (phase == 3) a = 2;
	else if (phase == 2) a = 3;

	bitClear(gMotor,bitEnable);  // enable motor;
	bitWrite(gMotor,bitA,a & 1);
	bitWrite(gMotor,bitB,(a >> 1) & 1);
}

void StepperMotor::setMotorBitsFromPhaseGo(void)
{
	setMotorBitsFromPhase();
	sendMotor();
}

void StepperMotor::goToStart(void)
{
	setMotorBitsFromPhaseGo();
	state = E_START;
	timeNextStateChange = millis() + msToStart;
}

void StepperMotor::stepPlus(const uint8_t delay)
{
	count--;
	position++;
	phase++;
	stepCommon(delay);
}

void StepperMotor::stepMinus(const uint8_t delay)
{
	count++;
	position--;
	phase--;
	stepCommon(delay);
}

void StepperMotor::stepCommon(const uint8_t delay)
{
	phase &= 0x3;
	timeNextStateChange = millis() + delay;
	setMotorBitsFromPhaseGo();
}


void StepperMotor::evaluate(void)
{
	unsigned long timeNow = millis();
	long int deltaTime = static_cast<long int>(timeNow - timeNextStateChange);
	if (deltaTime < 0 ) return;

	switch (state)
	{
	case E_START:
	case E_MOVE:
		if (!count)
		{
			state = E_STOP;
			timeNextStateChange = timeNow + msToHang;
		}
		else
		{
			state = E_MOVE;
			if (count > 0)
				stepPlus(slowSpeed);
			else
				stepMinus(slowSpeed);
		}
		break;
	case E_STOP:
		goToIdle();
		break;
	case E_HOME:
		{
			bool bState = digitalRead(pinOpto) == HIGH ? true : false;
			if (!bHomeHigh && bState)
				bHomeHigh = true;
			++homeCounter;
			if ((bHomeHigh && !bState) ||
				(!bHomeHigh && homeCounter > 20) ||
				(bHomeHigh && homeCounter > 300) )
			{
				count = 0;
				position = 0;
				state = E_MOVE;
			}
			else
			{
				stepPlus(highSpeed);
			}

		}
		break;
	case E_IDLE:
        if (desiredPosition != DoNothingDesiredPosition)
            count = desiredPosition - position;
	default:
		if (count)  // will start 
			goToStart();
		else
		{
			timeNextStateChange = timeNow + 2;
			state = E_IDLE;
		}
		break;
	}
}


void StepperMotor::home(void)
{
	digitalWrite(pinOpto,HIGH);  // inrease open drain voltage pull up
	state = E_HOME;
	bitSet(gMotor,bitLEDPwr);  // turn on LED power
	setMotorBitsFromPhaseGo();
	timeNextStateChange = millis() + msToStart;
	homeCounter = 0;
	bHomeHigh = false;
}
